@@ -13,6 +13,10 @@
 {-# LANGUAGE DataKinds #-}
 #endif
 
+#ifndef MIN_VERSION_hashable
+#define MIN_VERSION_hashable(x,y,z) 1
+#endif
+
 #ifndef MIN_VERSION_vector
 #define MIN_VERSION_vector(x,y,z) 1
 #endif
@@ -61,6 +65,9 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
 import Data.Serialize as Cereal
 import GHC.Arr (Ix(..))
 import qualified Data.Foldable as F
@@ -202,6 +209,10 @@ instance Foldable Quaternion where
   {-# INLINE foldMap #-}
   foldr f z (Quaternion e v) = f e (F.foldr f z v)
   {-# INLINE foldr #-}
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = False
+  length _ = 4
+#endif
 
 instance Traversable Quaternion where
   traverse f (Quaternion e v) = Quaternion <$> f e <*> traverse f v
@@ -258,6 +269,12 @@ instance Hashable a => Hashable (Quaternion a) where
   hashWithSalt s (Quaternion a b) = s `hashWithSalt` a `hashWithSalt` b
   {-# INLINE hashWithSalt #-}
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 Quaternion where
+  liftHashWithSalt h s (Quaternion a b) = liftHashWithSalt h (h s a) b
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 qNaN :: RealFloat a => Quaternion a
 qNaN = Quaternion fNaN (V3 fNaN fNaN fNaN) where fNaN = 0/0
 {-# INLINE qNaN #-}
@@ -358,9 +375,9 @@ pow q t = exp (t *^ log q)
 {-# INLINE pow #-}
 
 sqrte2pqiq :: (Floating a, Ord a) => a -> a -> a
-sqrte2pqiq e qiq -- = sqrt (e*e) + qiq
+sqrte2pqiq e qiq -- = sqrt (e*e + qiq)
   | e < - 1.5097698010472593e153 = -(qiq/e) - e
-  | e < 5.582399551122541e57      = sqrt ((e*e) + qiq) -- direct definition
+  | e < 5.582399551122541e57      = sqrt (e*e + qiq) -- direct definition
   | otherwise                     = (qiq/e) + e
 -- {-# SPECIALIZE sqrte2pqiq :: Double -> Double -> Double #-}
 -- {-# SPECIALIZE sqrte2pqiq :: Float -> Float -> Float #-}
@@ -391,11 +408,11 @@ instance RealFloat a => Floating (Quaternion a) where
     | ai <- sqrt qiq, exe <- exp e = reimagine (exe * cos ai) (exe * (sin ai / ai)) q
     where qiq = qi q
   {-# INLINE exp #-}
-  log q@(Quaternion e v@(V3 _i j k))
+  log q@(Quaternion e v)
     | qiq == 0 = if e >= 0
-                 then Quaternion (log e) v
-                 else Quaternion (log (negate e)) (V3 pi j k) -- mmm, pi
-    | ai <- sqrt qiq = reimagine (log m) (atan2 m e / ai) q
+                 then Quaternion (log e) v                   -- Using v rather than 0 preserves negative zeros
+                 else Quaternion (negate (log (negate e))) v -- negative scalar: negate quaternion, take log, negate again, preserves negative zeros
+    | ai <- sqrt qiq = reimagine (log m) (acos (e / m) / ai) q
     where qiq = qi q
           m = sqrte2pqiq e qiq
   {-# INLINE log #-}
@@ -416,7 +433,6 @@ instance RealFloat a => Floating (Quaternion a) where
   cos q@(Quaternion e v)
     | qiq == 0 = Quaternion (cos e) v
     | ai <- sqrt qiq = reimagine (cos e * cosh ai) (- sin e / ai / sinh ai) q -- 0.15 bits error
-                    -- reimagine (cos e * cosh ai) (- sin e * sinh ai / ai) q -- 13.5 bits worse
     where qiq = qi q
   {-# INLINE cos #-}
 
@@ -655,3 +671,15 @@ instance Ord1 Quaternion where compare1 = compare
 instance Show1 Quaternion where showsPrec1 = showsPrec
 instance Read1 Quaternion where readsPrec1 = readsPrec
 #endif
+
+instance Field1 (Quaternion a) (Quaternion a) a a where
+  _1 f (Quaternion w xyz) = f w <&> \w' -> Quaternion w' xyz
+
+instance Field2 (Quaternion a) (Quaternion a) a a where
+  _2 f (Quaternion w (V3 x y z)) = f x <&> \x' -> Quaternion w (V3 x' y z)
+
+instance Field3 (Quaternion a) (Quaternion a) a a where
+  _3 f (Quaternion w (V3 x y z)) = f y <&> \y' -> Quaternion w (V3 x y' z)
+
+instance Field4 (Quaternion a) (Quaternion a) a a where
+  _4 f (Quaternion w (V3 x y z)) = f z <&> \z' -> Quaternion w (V3 x y z')
