@@ -56,8 +56,13 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
 import Data.Ix
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
+#endif
 import Data.Serialize -- cereal
 #if __GLASGOW_HASKELL__ >= 707
 import qualified Data.Vector as V
@@ -132,6 +137,10 @@ instance Functor V0 where
 instance Foldable V0 where
   foldMap _ V0 = mempty
   {-# INLINE foldMap #-}
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = True
+  length _ = 0
+#endif
 
 instance Traversable V0 where
   traverse _ V0 = pure V0
@@ -243,6 +252,12 @@ instance Hashable (V0 a) where
   hashWithSalt s V0 = s
   {-# INLINE hashWithSalt #-}
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 V0 where
+  liftHashWithSalt _ s V0 = s
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 instance Epsilon (V0 a) where
   nearZero _ = True
   {-# INLINE nearZero #-}
