@@ -61,6 +61,9 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
 import Data.Semigroup.Foldable
 #if __GLASGOW_HASKELL__ >= 707
 import qualified Data.Vector as V
@@ -108,7 +111,7 @@ import qualified Data.Vector.Unboxed.Base as U
 --data V2 a = V2 !a !a deriving (Eq,Ord,Show,Read,Data,Typeable)
 newtype V1 a = V1 a
   deriving (Eq,Ord,Show,Read,Data,Typeable,
-            Functor,Foldable,Traversable,
+            Functor,Traversable,
             Epsilon,Storable,NFData
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 702
            ,Generic
@@ -118,6 +121,13 @@ newtype V1 a = V1 a
 #endif
            )
 
+instance Foldable V1 where
+  foldMap f (V1 a) = f a
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = False
+  length _ = 1
+#endif
+
 #if __GLASGOW_HASKELL__ >= 707
 instance Finite V1 where
   type Size V1 = 1
@@ -229,6 +239,12 @@ instance Hashable a => Hashable (V1 a) where
 #endif
   hashWithSalt s (V1 a) = s `hashWithSalt` a
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 V1 where
+  liftHashWithSalt h s (V1 a) = h s a
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 instance Metric V1 where
   dot (V1 a) (V1 b) = a * b
   {-# INLINE dot #-}
@@ -381,3 +397,6 @@ instance Ord1 V1 where compare1 = compare
 instance Show1 V1 where showsPrec1 = showsPrec
 instance Read1 V1 where readsPrec1 = readsPrec
 #endif
+
+instance Field1 (V1 a) (V1 b) a b where
+  _1 f (V1 x) = V1 <$> f x
