@@ -13,6 +13,10 @@
 {-# LANGUAGE DataKinds #-}
 #endif
 
+#ifndef MIN_VERSION_hashable
+#define MIN_VERSION_hashable(x,y,z) 1
+#endif
+
 #ifndef MIN_VERSION_vector
 #define MIN_VERSION_vector(x,y,z) 1
 #endif
@@ -59,6 +63,9 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
 import Data.Semigroup
 import Data.Semigroup.Foldable
 import Data.Serialize as Cereal
@@ -129,6 +136,10 @@ instance Functor V2 where
 instance Foldable V2 where
   foldMap f (V2 a b) = f a `mappend` f b
   {-# INLINE foldMap #-}
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = False
+  length _ = 2
+#endif
 
 instance Traversable V2 where
   traverse f (V2 a b) = V2 <$> f a <*> f b
@@ -156,6 +167,12 @@ instance Hashable a => Hashable (V2 a) where
   hashWithSalt s (V2 a b) = s `hashWithSalt` a `hashWithSalt` b
   {-# INLINE hashWithSalt #-}
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 V2 where
+  liftHashWithSalt h s (V2 a b) = s `h` a `h` b
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 instance Additive V2 where
   zero = pure 0
   {-# INLINE zero #-}
@@ -455,3 +472,9 @@ instance Ord1 V2 where compare1 = compare
 instance Show1 V2 where showsPrec1 = showsPrec
 instance Read1 V2 where readsPrec1 = readsPrec
 #endif
+
+instance Field1 (V2 a) (V2 a) a a where
+  _1 f (V2 x y) = f x <&> \x' -> V2 x' y
+
+instance Field2 (V2 a) (V2 a) a a where
+  _2 f (V2 x y) = f y <&> \y' -> V2 x y'
