@@ -13,6 +13,10 @@
 {-# LANGUAGE DataKinds #-}
 #endif
 
+#ifndef MIN_VERSION_hashable
+#define MIN_VERSION_hashable(x,y,z) 1
+#endif
+
 #ifndef MIN_VERSION_vector
 #define MIN_VERSION_vector(x,y,z) 1
 #endif
@@ -59,7 +63,12 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
+#endif
 import Data.Semigroup.Foldable
 import Data.Serialize as Cereal -- cereal
 #if __GLASGOW_HASKELL__ >= 707
@@ -99,7 +108,7 @@ data V3 a = V3 !a !a !a deriving (Eq,Ord,Show,Read,Data,Typeable
 
 #if __GLASGOW_HASKELL__ >= 707
 instance Finite V3 where
-  type Size V3 = 3 
+  type Size V3 = 3
   toV (V3 a b c) = V (V.fromListN 3 [a,b,c])
   fromV (V v) = V3 (v V.! 0) (v V.! 1) (v V.! 2)
 #endif
@@ -113,6 +122,10 @@ instance Functor V3 where
 instance Foldable V3 where
   foldMap f (V3 a b c) = f a `mappend` f b `mappend` f c
   {-# INLINE foldMap #-}
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = False
+  length _ = 3
+#endif
 
 instance Traversable V3 where
   traverse f (V3 a b c) = V3 <$> f a <*> f b <*> f c
@@ -226,6 +239,12 @@ instance Hashable a => Hashable (V3 a) where
   hashWithSalt s (V3 a b c) = s `hashWithSalt` a `hashWithSalt` b `hashWithSalt` c
   {-# INLINE hashWithSalt #-}
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 V3 where
+  liftHashWithSalt h s (V3 a b c) = s `h` a `h` b `h` c
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 instance Metric V3 where
   dot (V3 a b c) (V3 d e f) = a * d + b * e + c * f
   {-# INLINABLE dot #-}
@@ -475,3 +494,12 @@ instance Ord1 V3 where compare1 = compare
 instance Show1 V3 where showsPrec1 = showsPrec
 instance Read1 V3 where readsPrec1 = readsPrec
 #endif
+
+instance Field1 (V3 a) (V3 a) a a where
+  _1 f (V3 x y z) = f x <&> \x' -> V3 x' y z
+
+instance Field2 (V3 a) (V3 a) a a where
+  _2 f (V3 x y z) = f y <&> \y' -> V3 x y' z
+
+instance Field3 (V3 a) (V3 a) a a where
+  _3 f (V3 x y z) = f z <&> \z' -> V3 x y z'
