@@ -13,6 +13,10 @@
 {-# LANGUAGE DataKinds #-}
 #endif
 
+#ifndef MIN_VERSION_hashable
+#define MIN_VERSION_hashable(x,y,z) 1
+#endif
+
 #ifndef MIN_VERSION_vector
 #define MIN_VERSION_vector(x,y,z) 1
 #endif
@@ -65,7 +69,12 @@ import Data.Functor.Bind
 import Data.Functor.Classes
 import Data.Functor.Rep
 import Data.Hashable
+#if (MIN_VERSION_hashable(1,2,5))
+import Data.Hashable.Lifted
+#endif
+#if !(MIN_VERSION_base(4,11,0))
 import Data.Semigroup
+#endif
 import Data.Semigroup.Foldable
 import Data.Serialize as Cereal
 #if __GLASGOW_HASKELL__ >= 707
@@ -120,6 +129,10 @@ instance Functor V4 where
 instance Foldable V4 where
   foldMap f (V4 a b c d) = f a `mappend` f b `mappend` f c `mappend` f d
   {-# INLINE foldMap #-}
+#if __GLASGOW_HASKELL__ >= 710
+  null _ = False
+  length _ = 4
+#endif
 
 instance Traversable V4 where
   traverse f (V4 a b c d) = V4 <$> f a <*> f b <*> f c <*> f d
@@ -246,6 +259,12 @@ instance Hashable a => Hashable (V4 a) where
   hashWithSalt s (V4 a b c d) = s `hashWithSalt` a `hashWithSalt` b `hashWithSalt` c `hashWithSalt` d
   {-# INLINE hashWithSalt #-}
 
+#if (MIN_VERSION_hashable(1,2,5))
+instance Hashable1 V4 where
+  liftHashWithSalt h s (V4 a b c d) = s `h` a `h` b `h` c `h` d
+  {-# INLINE liftHashWithSalt #-}
+#endif
+
 -- | A space that distinguishes orthogonal basis vectors '_x', '_y', '_z', '_w'. (It may have more.)
 class R3 t => R4 t where
   -- |
@@ -613,3 +632,15 @@ instance Ord1 V4 where compare1 = compare
 instance Show1 V4 where showsPrec1 = showsPrec
 instance Read1 V4 where readsPrec1 = readsPrec
 #endif
+
+instance Field1 (V4 a) (V4 a) a a where
+  _1 f (V4 x y z w) = f x <&> \x' -> V4 x' y z w
+
+instance Field2 (V4 a) (V4 a) a a where
+  _2 f (V4 x y z w) = f y <&> \y' -> V4 x y' z w
+
+instance Field3 (V4 a) (V4 a) a a where
+  _3 f (V4 x y z w) = f z <&> \z' -> V4 x y z' w
+
+instance Field4 (V4 a) (V4 a) a a where
+  _4 f (V4 x y z w) = f w <&> \w' -> V4 x y z w'
